@@ -127,7 +127,7 @@ static TYPELIB command_typelib=
 static struct my_option my_long_options[] =
 {
 #ifdef __NETWARE__
-  {"autoclose", 'a', " Auto close the screen on exit for NetWare",
+  {"auto-close", OPT_AUTO_CLOSE, "Auto close the screen on exit for Netware.",
    0, 0, 0, GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0},
 #endif
   {"count", 'c',
@@ -227,9 +227,9 @@ get_one_option(int optid, const struct my_option *opt __attribute__((unused)),
 
   switch(optid) {
 #ifdef __NETWARE__
-  case 'a':
-   setscreenmode(SCR_AUTOCLOSE_ON_EXIT);      // auto close the screen /
-   break;
+  case OPT_AUTO_CLOSE:
+    setscreenmode(SCR_AUTOCLOSE_ON_EXIT);
+    break;
 #endif
   case 'c':
     opt_count_iterations= 1;
@@ -271,7 +271,7 @@ get_one_option(int optid, const struct my_option *opt __attribute__((unused)),
 	option_wait=1;
     }
     else
-      option_wait= ~0;
+      option_wait= ~(uint)0;
     break;
   case '?':
   case 'I':					/* Info */
