@@ -16,9 +16,11 @@
 
 
 #ifndef MYSQL_CLIENT
-#ifdef __GNUC__
+
+#ifdef USE_PRAGMA_IMPLEMENTATION
 #pragma implementation				// gcc: Class implementation
 #endif
+
 #include  "mysql_priv.h"
 #include "slave.h"
 #include <my_dir.h>
@@ -205,7 +207,7 @@ static inline int read_str(char * &buf, char *buf_end, char * &str,
 /*
   Transforms a string into "" or its expression in 0x... form.
 */
-static char *str_to_hex(char *to, char *from, uint len)
+char *str_to_hex(char *to, const char *from, uint len)
 {
   char *p= to;
   if (len)
@@ -1006,6 +1008,16 @@ int Query_log_event::exec_event(struct st_relay_log_info* rli)
 #endif
   clear_all_errors(thd, rli);
 
+  /*
+    Note:   We do not need to execute reset_one_shot_variables() if this
+            db_ok() test fails.
+    Reason: The db stored in binlog events is the same for SET and for
+            its companion query.  If the SET is ignored because of
+            db_ok(), the companion query will also be ignored, and if
+            the companion query is ignored in the db_ok() test of
+            ::exec_event(), then the companion SET also have so we
+            don't need to reset_one_shot_variables().
+  */
   if (db_ok(thd->db, replicate_do_db, replicate_ignore_db))
   {
     thd->set_time((time_t)when);
@@ -1760,6 +1772,16 @@ int Load_log_event::exec_event(NET* net, struct st_relay_log_info* rli,
     Create_file_log_event::exec_event() and then discarding Append_block and
     al. Another way is do the filtering in the I/O thread (more efficient: no
     disk writes at all).
+
+
+    Note:   We do not need to execute reset_one_shot_variables() if this
+            db_ok() test fails.
+    Reason: The db stored in binlog events is the same for SET and for
+            its companion query.  If the SET is ignored because of
+            db_ok(), the companion query will also be ignored, and if
+            the companion query is ignored in the db_ok() test of
+            ::exec_event(), then the companion SET also have so we
+            don't need to reset_one_shot_variables().
   */
   if (db_ok(thd->db, replicate_do_db, replicate_ignore_db))
   {
@@ -1807,8 +1829,11 @@ int Load_log_event::exec_event(NET* net, struct st_relay_log_info* rli,
                                           "` <...>", NullS) - load_data_query);
         thd->query= load_data_query;
       }
+
       if (sql_ex.opt_flags & REPLACE_FLAG)
+      {
 	handle_dup= DUP_REPLACE;
+      }
       else if (sql_ex.opt_flags & IGNORE_FLAG)
       {
         ignore= 1;
@@ -1831,6 +1856,16 @@ int Load_log_event::exec_event(NET* net, struct st_relay_log_info* rli,
         */
         handle_dup= DUP_ERROR;
       }
+      /*
+        We need to set thd->lex->sql_command and thd->lex->duplicates
+        since InnoDB tests these variables to decide if this is a LOAD
+        DATA ... REPLACE INTO ... statement even though mysql_parse()
+        is not called.  This is not needed in 5.0 since there the LOAD
+        DATA ... statement is replicated using mysql_parse(), which
+        sets the thd->lex fields correctly.
+      */
+      thd->lex->sql_command= SQLCOM_LOAD;
+      thd->lex->duplicates= handle_dup;
 
       sql_exchange ex((char*)fname, sql_ex.opt_flags & DUMPFILE_FLAG);
       String field_term(sql_ex.field_term,sql_ex.field_term_len,log_cs);
