@@ -25,14 +25,16 @@
 #include <direct.h>
 #endif
 
+#define MAX_DROP_TABLE_Q_LEN      1024
+
 const char *del_exts[]= {".frm", ".BAK", ".TMD",".opt", NullS};
 static TYPELIB deletable_extentions=
 {array_elements(del_exts)-1,"del_exts", del_exts, NULL};
 
 static long mysql_rm_known_files(THD *thd, MY_DIR *dirp,
-				 const char *db, const char *path,
-				 uint level);
-
+				 const char *db, const char *path, uint level, 
+                                 TABLE_LIST **dropped_tables);
+         
 /* Database options hash */
 static HASH dboptions;
 static my_bool dboptions_init= 0;
@@ -50,6 +52,7 @@ typedef struct my_dbopt_st
 /*
   Function we use in the creation of our hash to get key.
 */
+
 static byte* dboptions_get_key(my_dbopt_t *opt, uint *length,
                                my_bool not_used __attribute__((unused)))
 {
@@ -58,6 +61,21 @@ static byte* dboptions_get_key(my_dbopt_t *opt, uint *length,
 }
 
 
+/*
+  Helper function to write a query to binlog used by mysql_rm_db()
+*/
+
+static inline void write_to_binlog(THD *thd, char *query, uint q_len,
+                                   char *db, uint db_len)
+{
+  Query_log_event qinfo(thd, query, q_len, 0, 0);
+  qinfo.error_code= 0;
+  qinfo.db= db;
+  qinfo.db_len= db_len;
+  mysql_bin_log.write(&qinfo);
+}  
+
+
 /*
   Function to free dboptions hash element
 */
@@ -409,7 +427,11 @@ int mysql_create_db(THD *thd, char *db, HA_CREATE_INFO *create_info,
       error= -1;
       goto exit;
     }
-    result= 0;
+    push_warning_printf(thd, MYSQL_ERROR::WARN_LEVEL_NOTE,
+                        ER_DB_CREATE_EXISTS, ER(ER_DB_CREATE_EXISTS), db);
+    error= 0;
+    send_ok(thd);
+    goto exit;
   }
   else
   {
@@ -585,6 +607,7 @@ int mysql_rm_db(THD *thd,char *db,bool if_exists, bool silent)
   char	path[FN_REFLEN+16], tmp_db[NAME_LEN+1];
   MY_DIR *dirp;
   uint length;
+  TABLE_LIST* dropped_tables= 0;
   DBUG_ENTER("mysql_rm_db");
 
   VOID(pthread_mutex_lock(&LOCK_mysql_create_db));
@@ -621,8 +644,10 @@ int mysql_rm_db(THD *thd,char *db,bool if_exists, bool silent)
     remove_db_from_cache(db);
     pthread_mutex_unlock(&LOCK_open);
 
+    
     error= -1;
-    if ((deleted= mysql_rm_known_files(thd, dirp, db, path, 0)) >= 0)
+    if ((deleted= mysql_rm_known_files(thd, dirp, db, path, 0,
+                                       &dropped_tables)) >= 0)
     {
       ha_drop_database(path);
       query_cache_invalidate1(db);
@@ -672,6 +697,41 @@ int mysql_rm_db(THD *thd,char *db,bool if_exists, bool silent)
     send_ok(thd, (ulong) deleted);
     thd->server_status&= ~SERVER_STATUS_DB_DROPPED;
   }
+  else if (mysql_bin_log.is_open())
+  {
+    char *query, *query_pos, *query_end, *query_data_start;
+    TABLE_LIST *tbl;
+    uint db_len;
+
+    if (!(query= thd->alloc(MAX_DROP_TABLE_Q_LEN)))
+      goto exit; /* not much else we can do */
+    query_pos= query_data_start= strmov(query,"drop table ");
+    query_end= query + MAX_DROP_TABLE_Q_LEN;
+    db_len= strlen(db);
+
+    for (tbl= dropped_tables; tbl; tbl= tbl->next)
+    {
+      uint tbl_name_len;
+
+      /* 3 for the quotes and the comma*/
+      tbl_name_len= strlen(tbl->real_name) + 3;
+      if (query_pos + tbl_name_len + 1 >= query_end)
+      {
+        write_to_binlog(thd, query, query_pos -1 - query, db, db_len);
+        query_pos= query_data_start;
+      }
+
+      *query_pos++ = '`';
+      query_pos= strmov(query_pos,tbl->real_name);
+      *query_pos++ = '`';
+      *query_pos++ = ',';
+    }
+
+    if (query_pos != query_data_start)
+    {
+      write_to_binlog(thd, query, query_pos -1 - query, db, db_len);
+    }
+  }
 
 exit:
   start_waiting_global_read_lock(thd);
@@ -716,7 +776,8 @@ exit2:
 */
 
 static long mysql_rm_known_files(THD *thd, MY_DIR *dirp, const char *db,
-				 const char *org_path, uint level)
+				 const char *org_path, uint level,
+                                 TABLE_LIST **dropped_tables)
 {
   long deleted=0;
   ulong found_other_files=0;
@@ -758,7 +819,7 @@ static long mysql_rm_known_files(THD *thd, MY_DIR *dirp, const char *db,
       if ((new_dirp = my_dir(newpath,MYF(MY_DONT_SORT))))
       {
 	DBUG_PRINT("my",("New subdir found: %s", newpath));
-	if ((mysql_rm_known_files(thd, new_dirp, NullS, newpath,1)) < 0)
+	if ((mysql_rm_known_files(thd, new_dirp, NullS, newpath,1,0)) < 0)
 	  goto err;
 	if (!(copy_of_path= thd->memdup(newpath, length+1)) ||
 	    !(dir= new (thd->mem_root) String(copy_of_path, length,
@@ -818,6 +879,9 @@ static long mysql_rm_known_files(THD *thd, MY_DIR *dirp, const char *db,
   }
   my_dirend(dirp);  
   
+  if (dropped_tables)
+    *dropped_tables= tot_list;
+  
   /*
     If the directory is a symbolic link, remove the link first, then
     remove the directory the symbolic link pointed at
@@ -946,7 +1010,7 @@ bool mysql_change_db(THD *thd, const char *name)
   length=unpack_dirname(path,path);		// Convert if not unix
   if (length && path[length-1] == FN_LIBCHAR)
     path[length-1]=0;				// remove ending '\'
-  if (access(path,F_OK))
+  if (my_access(path,F_OK))
   {
     net_printf(thd,ER_BAD_DB_ERROR,dbname);
     my_free(dbname,MYF(0));
