@@ -27,8 +27,8 @@
 #include "ha_innodb.h"
 #include "sql_select.h"
 
-int mysql_delete(THD *thd, TABLE_LIST *table_list, COND *conds, SQL_LIST *order,
-                 ha_rows limit, ulong options)
+int mysql_delete(THD *thd, TABLE_LIST *table_list, COND *conds,
+                 SQL_LIST *order, ha_rows limit, ulong options)
 {
   int		error;
   TABLE		*table;
@@ -117,6 +117,7 @@ int mysql_delete(THD *thd, TABLE_LIST *table_list, COND *conds, SQL_LIST *order,
 
     bzero((char*) &tables,sizeof(tables));
     tables.table = table;
+    tables.alias = table_list->alias;
 
     table->sort.io_cache = (IO_CACHE *) my_malloc(sizeof(IO_CACHE),
                                              MYF(MY_FAE | MY_ZEROFILL));
@@ -265,6 +266,7 @@ int mysql_prepare_delete(THD *thd, TABLE_LIST *table_list, Item **conds)
 				  select_lex.table_list.first);
   DBUG_ENTER("mysql_prepare_delete");
 
+  thd->allow_sum_func= 0;
   if (setup_conds(thd, delete_table_list, conds) || 
       setup_ftfuncs(&thd->lex->select_lex))
     DBUG_RETURN(-1);
@@ -616,6 +618,8 @@ int mysql_truncate(THD *thd, TABLE_LIST *table_list, bool dont_send_ok)
     TABLE *table= *table_ptr;
     table->file->info(HA_STATUS_AUTO | HA_STATUS_NO_LOCK);
     db_type table_type=table->db_type;
+    if (!ha_supports_generate(table_type))
+      goto trunc_by_del;
     strmov(path,table->path);
     *table_ptr= table->next;			// Unlink table from list
     close_temporary(table,0);
@@ -634,7 +638,7 @@ int mysql_truncate(THD *thd, TABLE_LIST *table_list, bool dont_send_ok)
 
   (void) sprintf(path,"%s/%s/%s%s",mysql_data_home,table_list->db,
 		 table_list->real_name,reg_ext);
-  fn_format(path,path,"","",4);
+  fn_format(path, path, "", "", MY_UNPACK_FILENAME);
 
   if (!dont_send_ok)
   {
@@ -646,15 +650,7 @@ int mysql_truncate(THD *thd, TABLE_LIST *table_list, bool dont_send_ok)
       DBUG_RETURN(-1);
     }
     if (!ha_supports_generate(table_type))
-    {
-      /* Probably InnoDB table */
-      table_list->lock_type= TL_WRITE;
-      ha_enable_transaction(thd, FALSE);
-      error= mysql_delete(thd, table_list, (COND*) 0, (SQL_LIST*) 0,
-			  HA_POS_ERROR, 0);
-      ha_enable_transaction(thd, TRUE);
-      DBUG_RETURN(error);
-    }
+      goto trunc_by_del;
     if (lock_and_wait_for_table_name(thd, table_list))
       DBUG_RETURN(-1);
   }
@@ -689,4 +685,16 @@ end:
     VOID(pthread_mutex_unlock(&LOCK_open));
   }
   DBUG_RETURN(error ? -1 : 0);
+
+ trunc_by_del:
+  /* Probably InnoDB table */
+  ulong save_options= thd->options;
+  table_list->lock_type= TL_WRITE;
+  thd->options&= ~(ulong) (OPTION_BEGIN | OPTION_NOT_AUTOCOMMIT);
+  ha_enable_transaction(thd, FALSE);
+  error= mysql_delete(thd, table_list, (COND*) 0, (SQL_LIST*) 0,
+                      HA_POS_ERROR, 0);
+  ha_enable_transaction(thd, TRUE);
+  thd->options= save_options;
+  DBUG_RETURN(error);
 }
