@@ -170,7 +170,8 @@ cd ..
 BASEDIR=`pwd`
 cd $CWD
 MYSQL_TEST_DIR=$BASEDIR/mysql-test
-export MYSQL_TEST_DIR
+MYSQL_TEST_WINDIR=$MYSQL_TEST_DIR
+export MYSQL_TEST_DIR MYSQL_TEST_WINDIR
 STD_DATA=$MYSQL_TEST_DIR/std_data
 hostname=`hostname`		# Installed in the mysql privilege table
 
@@ -569,10 +570,7 @@ if [ x$SOURCE_DIST = x1 ] ; then
  NDB_MGM="$BASEDIR/ndb/src/mgmclient/ndb_mgm"
 
  if [ -n "$USE_PURIFY" ] ; then
-   PSUP="$MYSQL_TEST_DIR/purify.suppress"
-   echo "suppress UMR rw_read_held; mi_open; ha_myisam::open64; handler::ha_open; openfrm" >  $PSUP
-   echo "suppress UMR my_end; main" >> $PSUP
-   echo "suppress UMR _doprnt; fprintf; my_end; main" >> $PSUP
+   PSUP="$MYSQL_TEST_DIR/suppress.purify"
    PURIFYOPTIONS="-windows=no -log-file=%v.purifylog -append-logfile -add-suppression-files=$PSUP"
    if [ -f "${MYSQL_TEST}-purify" ] ; then
      MYSQL_TEST="${MYSQL_TEST}-purify"
@@ -691,10 +689,13 @@ MYSQL_CLIENT_TEST="$MYSQL_CLIENT_TEST --no-defaults --testcase --user=root --soc
 if [ "x$USE_EMBEDDED_SERVER" = "x1" ]; then
   MYSQL_CLIENT_TEST="$MYSQL_CLIENT_TEST -A --language=$LANGUAGE -A --datadir=$SLAVE_MYDDIR -A --character-sets-dir=$CHARSETSDIR"
 fi
+# Save path and name of mysqldump
+MYSQL_DUMP_DIR="$MYSQL_DUMP"
+export MYSQL_DUMP_DIR
 MYSQL_DUMP="$MYSQL_DUMP --no-defaults -uroot --socket=$MASTER_MYSOCK --password=$DBPASSWD $EXTRA_MYSQLDUMP_OPT"
 MYSQL_BINLOG="$MYSQL_BINLOG --no-defaults --local-load=$MYSQL_TMP_DIR $EXTRA_MYSQLBINLOG_OPT"
 MYSQL_FIX_SYSTEM_TABLES="$MYSQL_FIX_SYSTEM_TABLES --no-defaults --host=localhost --port=$MASTER_MYPORT --socket=$MASTER_MYSOCK --user=root --password=$DBPASSWD --basedir=$BASEDIR --bindir=$CLIENT_BINDIR --verbose"
-MYSQL="$MYSQL --host=localhost --port=$MASTER_MYPORT --socket=$MASTER_MYSOCK --user=root --password=$DBPASSWD"
+MYSQL="$MYSQL --no-defaults --host=localhost --port=$MASTER_MYPORT --socket=$MASTER_MYSOCK --user=root --password=$DBPASSWD"
 export MYSQL MYSQL_DUMP MYSQL_BINLOG MYSQL_FIX_SYSTEM_TABLES
 export CLIENT_BINDIR MYSQL_CLIENT_TEST CHARSETSDIR
 export NDB_TOOLS_DIR
@@ -713,6 +714,7 @@ if [ x$USE_TIMER = x1 ] ; then
 fi
 MYSQL_TEST_BIN=$MYSQL_TEST
 MYSQL_TEST="$MYSQL_TEST $MYSQL_TEST_ARGS"
+export MYSQL_TEST
 GDB_CLIENT_INIT=$MYSQL_TMP_DIR/gdbinit.client
 GDB_MASTER_INIT=$MYSQL_TMP_DIR/gdbinit.master
 GDB_SLAVE_INIT=$MYSQL_TMP_DIR/gdbinit.slave
@@ -1184,8 +1186,8 @@ start_master()
            $NOT_FIRST_MASTER_EXTRA_OPTS"
   fi
 
-  CUR_MYERR=$MASTER_MYERR
-  CUR_MYSOCK=$MASTER_MYSOCK
+  CUR_MYERR=$MASTER_MYERR$1
+  CUR_MYSOCK=$MASTER_MYSOCK$1
 
   # For embedded server we collect the server flags and return
   if [ "x$USE_EMBEDDED_SERVER" = "x1" ] ; then
@@ -1660,11 +1662,11 @@ run_testcase ()
       $ECHO "$RES$RES_SPACE [ pass ]   $TIMER"
     else
       # why the following ``if'' ? That is why res==1 is special ?
-      if [ $res = 2 ]; then
+      if [ $res = 62 ]; then
         skip_inc
 	$ECHO "$RES$RES_SPACE [ skipped ]"
       else
-        if [ $res -gt 2 ]; then
+        if [ $res -ne 1 ]; then
           $ECHO "mysqltest returned unexpected code $res, it has probably crashed" >> $TIMEFILE
         fi
 	total_inc
