@@ -383,26 +383,28 @@ static my_bool init_available_charsets(myf myflags)
       while we may changing the cs_info_table
     */
     pthread_mutex_lock(&THR_LOCK_charset);
-
-    bzero(&all_charsets,sizeof(all_charsets));
-    init_compiled_charsets(myflags);
-    
-    /* Copy compiled charsets */
-    for (cs=all_charsets;
-	 cs < all_charsets+array_elements(all_charsets)-1 ;
-	 cs++)
+    if (!charset_initialized)
     {
-      if (*cs)
+      bzero(&all_charsets,sizeof(all_charsets));
+      init_compiled_charsets(myflags);
+      
+      /* Copy compiled charsets */
+      for (cs=all_charsets;
+           cs < all_charsets+array_elements(all_charsets)-1 ;
+           cs++)
       {
-        if (cs[0]->ctype)
-          if (init_state_maps(*cs))
-            *cs= NULL;
+        if (*cs)
+        {
+          if (cs[0]->ctype)
+            if (init_state_maps(*cs))
+              *cs= NULL;
+        }
       }
+      
+      strmov(get_charsets_dir(fname), MY_CHARSET_INDEX);
+      error= my_read_charset_file(fname,myflags);
+      charset_initialized=1;
     }
-    
-    strmov(get_charsets_dir(fname), MY_CHARSET_INDEX);
-    error= my_read_charset_file(fname,myflags);
-    charset_initialized=1;
     pthread_mutex_unlock(&THR_LOCK_charset);
   }
   return error;
@@ -635,3 +637,29 @@ ulong escape_string_for_mysql(CHARSET_INFO *charset_info, char *to,
   *to= 0;
   return (ulong) (to - to_start);
 }
+
+
+#ifdef BACKSLASH_MBTAIL
+static CHARSET_INFO *fs_cset_cache= NULL;
+
+CHARSET_INFO *fs_character_set()
+{
+  if (!fs_cset_cache)
+  {
+    char buf[10]= "cp";
+    GetLocaleInfo(LOCALE_SYSTEM_DEFAULT, LOCALE_IDEFAULTANSICODEPAGE,
+                  buf+2, sizeof(buf)-3);
+    /*
+      We cannot call get_charset_by_name here
+      because fs_character_set() is executed before
+      LOCK_THD_charset mutex initialization, which
+      is used inside get_charset_by_name.
+      As we're now interested in cp932 only,
+      let's just detect it using strcmp().
+    */
+    fs_cset_cache= !strcmp(buf, "cp932") ?
+                   &my_charset_cp932_japanese_ci : &my_charset_bin;
+  }
+  return fs_cset_cache;
+}
+#endif
