@@ -143,15 +143,6 @@ Ndb::init(int aMaxNoOfTransactions)
 error_handler:
   ndbout << "error_handler" << endl;
   releaseTransactionArrays();
-  while ( theConIdleList != NULL )
-    freeNdbCon();
-  while ( theSignalIdleList != NULL )
-    freeSignal();
-  while (theRecAttrIdleList != NULL)
-    freeRecAttr(); 
-  while (theOpIdleList != NULL)
-    freeOperation();
-  
   delete theDictionary;
   TransporterFacade::instance()->close(theNdbBlockNumber, 0);
   DBUG_RETURN(-1);
@@ -453,7 +444,7 @@ Ndb::handleReceivedSignal(NdbApiSignal* aSignal, LinearSectionPtr ptr[3])
       tFirstDataPtr = int2void(tFirstData);
       if(tFirstDataPtr != 0){
 	tOp = void2rec_op(tFirstDataPtr);
-	if (tOp->checkMagicNumber() == 0) {
+	if (tOp->checkMagicNumber(false) == 0) {
 	  tCon = tOp->theNdbCon;
 	  if (tCon != NULL) {
 	    if ((tCon->theSendStatus == NdbConnection::sendTC_OP) ||
@@ -466,11 +457,11 @@ Ndb::handleReceivedSignal(NdbApiSignal* aSignal, LinearSectionPtr ptr[3])
 	    }//if
 	  }//if
 	}//if
-      } else {
-#ifdef VM_TRACE
-	ndbout_c("Recevied TCKEY_FAILREF wo/ operation");
-#endif
       }
+#ifdef VM_TRACE
+      ndbout_c("Recevied TCKEY_FAILREF wo/ operation");
+#endif
+      return;
       break;
     }
   case GSN_TCKEYREF:
