@@ -55,9 +55,6 @@ typedef struct ErrorBundle {
 #define NI ndberror_cl_function_not_implemented
 #define UE ndberror_cl_unknown_error_code
 
-static const char REDO_BUFFER_MSG[]=
-"REDO log buffers overloaded, consult online manual (increase RedoBuffer, and|or decrease TimeBetweenLocalCheckpoints, and|or increase NoOfFragmentLogFiles)";
-
 static const char* empty_string = "";
 
 /*
@@ -164,8 +161,9 @@ ErrorBundle ErrorCodes[] = {
   { 830,  TR, "Out of add fragment operation records" },
   { 873,  TR, "Out of attrinfo records for scan in tuple manager" },
   { 1217, TR, "Out of operation records in local data manager (increase MaxNoOfLocalOperations)" },
-  { 1220, TR, REDO_BUFFER_MSG },
+  { 1220, TR, "REDO log files overloaded, consult online manual (decrease TimeBetweenLocalCheckpoints, and|or increase NoOfFragmentLogFiles)" },
   { 1222, TR, "Out of transaction markers in LQH" },
+  { 1224, TR, "Out of Send Buffer space in LQH" },
   { 4021, TR, "Out of Send Buffer space in NDB API" },
   { 4022, TR, "Out of Send Buffer space in NDB API" },
   { 4032, TR, "Out of Send Buffer space in NDB API" },
@@ -194,10 +192,10 @@ ErrorBundle ErrorCodes[] = {
   /**
    * OverloadError
    */
-  { 410,  OL, REDO_BUFFER_MSG },
+  { 410,  OL, "REDO log files overloaded, consult online manual (decrease TimeBetweenLocalCheckpoints, and|or increase NoOfFragmentLogFiles)" },
   { 677,  OL, "Index UNDO buffers overloaded (increase UndoIndexBuffer)" },
   { 891,  OL, "Data UNDO buffers overloaded (increase UndoDataBuffer)" },
-  { 1221, OL, REDO_BUFFER_MSG },
+  { 1221, OL, "REDO buffers overloaded, consult online manual (increase RedoBuffer)" },
   { 4006, OL, "Connect failure - out of connection objects (increase MaxNoOfConcurrentTransactions)" }, 
 
 
@@ -298,8 +296,8 @@ ErrorBundle ErrorCodes[] = {
   { 703,  SE, "Invalid table format" },
   { 704,  SE, "Attribute name too long" },
   { 705,  SE, "Table name too long" },
-  { 707,  SE, "No more table metadata records" },  
-  { 708,  SE, "No more attribute metadata records" },
+  { 707,  SE, "No more table metadata records (increase MaxNoOfTables)" },  
+  { 708,  SE, "No more attribute metadata records (increase MaxNoOfAttributes)" },
   { 709,  SE, "No such table existed" },
   { 721,  SE, "Table or index with given name already exists" },
   { 723,  SE, "No such table existed" },
@@ -312,6 +310,8 @@ ErrorBundle ErrorCodes[] = {
   { 742,  SE, "Unsupported attribute type in index" },
   { 743,  SE, "Unsupported character set in table or index" },
   { 744,  SE, "Character string is invalid for given character set" },
+  { 761,  SE, "Unable to drop table as backup is in progress" },
+  { 762,  SE, "Unable to alter table as backup is in progress" },
   { 241,  SE, "Invalid schema object version" },
   { 283,  SE, "Table is being dropped" },
   { 284,  SE, "Table not defined in transaction coordinator" },
