@@ -284,7 +284,8 @@ bool close_cached_tables(THD *thd, bool if_wait_for_refresh,
     bool found=0;
     for (TABLE_LIST *table=tables ; table ; table=table->next)
     {
-      if (remove_table_from_cache(thd, table->db, table->real_name, 1))
+      if (remove_table_from_cache(thd, table->db, table->real_name,
+                                  RTFC_OWNED_BY_THD_FLAG))
 	found=1;
     }
     if (!found)
@@ -485,58 +486,62 @@ void close_temporary(TABLE *table,bool delete_table)
 void close_temporary_tables(THD *thd)
 {
   TABLE *table,*next;
-  char *query, *name_in_query, *end;
-  uint greatest_key_length= 0;
+  char *query, *end;
+  uint query_buf_size; 
+  bool found_user_tables = 0;
 
   if (!thd->temporary_tables)
     return;
   
-  /*
-    We write a DROP TEMPORARY TABLE for each temp table left, so that our
-    replication slave can clean them up. Not one multi-table DROP TABLE binlog
-    event: this would cause problems if slave uses --replicate-*-table.
-  */
   LINT_INIT(end);
+  query_buf_size= 50;   // Enough for DROP ... TABLE IF EXISTS
 
-  /* We'll re-use always same buffer so make it big enough for longest name */
   for (table=thd->temporary_tables ; table ; table=table->next)
-    greatest_key_length= max(greatest_key_length, table->key_length);
+    /*
+      We are going to add 4 ` around the db/table names, so 1 does not look
+      enough; indeed it is enough, because table->key_length is greater (by 8,
+      because of server_id and thread_id) than db||table.
+    */
+    query_buf_size+= table->key_length+1;
 
-  if ((query = alloc_root(thd->mem_root, greatest_key_length+50)))
+  if ((query = alloc_root(thd->mem_root, query_buf_size)))
     // Better add "if exists", in case a RESET MASTER has been done
-    name_in_query= strmov(query, "DROP /*!40005 TEMPORARY */ TABLE IF EXISTS `");
+    end=strmov(query, "DROP /*!40005 TEMPORARY */ TABLE IF EXISTS ");
 
   for (table=thd->temporary_tables ; table ; table=next)
   {
-    /*
-      In we are OOM for 'query' this is not fatal. We skip temporary tables
-      not created directly by the user.
-    */
-    if (query && mysql_bin_log.is_open() && (table->real_name[0] != '#'))
+    if (query) // we might be out of memory, but this is not fatal
     {
+      // skip temporary tables not created directly by the user
+      if (table->real_name[0] != '#')
+	found_user_tables = 1;
       /*
         Here we assume table_cache_key always starts
         with \0 terminated db name
       */
-      end = strxmov(name_in_query, table->table_cache_key, "`.`",
-                    table->real_name, "`", NullS);
-      Query_log_event qinfo(thd, query, (ulong)(end-query), 0, FALSE);
-      /*
-        Imagine the thread had created a temp table, then was doing a SELECT, and
-        the SELECT was killed. Then it's not clever to mark the statement above as
-        "killed", because it's not really a statement updating data, and there
-        are 99.99% chances it will succeed on slave. And, if thread is
-        killed now, it's not clever either.
-        If a real update (one updating a persistent table) was killed on the
-        master, then this real update will be logged with error_code=killed,
-        rightfully causing the slave to stop.
-      */
-      qinfo.error_code= 0;
-      mysql_bin_log.write(&qinfo);
+      end = strxmov(end,"`",table->table_cache_key,"`.`",
+                    table->real_name,"`,", NullS);
     }
     next=table->next;
     close_temporary(table);
   }
+  if (query && found_user_tables && mysql_bin_log.is_open())
+  {
+    /* The -1 is to remove last ',' */
+    thd->clear_error();
+    Query_log_event qinfo(thd, query, (ulong)(end-query)-1, 0, FALSE);
+    /*
+      Imagine the thread had created a temp table, then was doing a SELECT, and
+      the SELECT was killed. Then it's not clever to mark the statement above as
+      "killed", because it's not really a statement updating data, and there
+      are 99.99% chances it will succeed on slave.
+      If a real update (one updating a persistent table) was killed on the
+      master, then this real update will be logged with error_code=killed,
+      rightfully causing the slave to stop.
+    */
+    qinfo.error_code= 0;
+    mysql_bin_log.write(&qinfo);
+  }
   thd->temporary_tables=0;
 }
 
@@ -863,6 +868,13 @@ TABLE *open_table(THD *thd,const char *db,const char *table_name,
   {
     if (table->version != refresh_version)
     {
+      if (! refresh)
+      {
+        /* Ignore flush for now, but force close after usage. */
+        thd->version= table->version;
+        continue;
+      }
+
       /*
       ** There is a refresh in progress for this table
       ** Wait until the table is freed or the thread is killed.
@@ -975,6 +987,7 @@ TABLE *open_table(THD *thd,const char *db,const char *table_name,
   if (table->timestamp_field)
     table->timestamp_field_type= table->timestamp_field->get_auto_set_type();
   DBUG_ASSERT(table->key_read == 0);
+  DBUG_ASSERT(table->insert_values == 0);
   DBUG_RETURN(table);
 }
 
@@ -1159,7 +1172,7 @@ bool reopen_tables(THD *thd,bool get_locks,bool in_refresh)
     MYSQL_LOCK *lock;
     /* We should always get these locks */
     thd->some_tables_deleted=0;
-    if ((lock=mysql_lock_tables(thd,tables,(uint) (tables_ptr-tables))))
+    if ((lock= mysql_lock_tables(thd, tables, (uint) (tables_ptr-tables), 0)))
     {
       thd->locked_tables=mysql_lock_merge(thd->locked_tables,lock);
     }
@@ -1263,6 +1276,7 @@ bool wait_for_tables(THD *thd)
   {
     /* Now we can open all tables without any interference */
     thd->proc_info="Reopen tables";
+    thd->version= refresh_version;
     result=reopen_tables(thd,0,0);
   }
   pthread_mutex_unlock(&LOCK_open);
@@ -1370,9 +1384,19 @@ static int open_unireg_entry(THD *thd, TABLE *entry, const char *db,
        trying to discover the table at the same time.
       */
       if (discover_retry_count++ != 0)
-       goto err;
-      if (ha_create_table_from_engine(thd, db, name, TRUE) != 0)
-       goto err;
+        goto err;
+      if (ha_create_table_from_engine(thd, db, name) > 0)
+      {
+        /* Give right error message */
+        thd->clear_error();
+        DBUG_PRINT("error", ("Dicovery of %s/%s failed", db, name));
+        my_printf_error(ER_UNKNOWN_ERROR,
+                        "Failed to open '%-.64s', error while "
+                        "unpacking from engine",
+                        MYF(0), name);
+
+        goto err;
+      }
 
       thd->clear_error(); // Clear error message
       continue;
@@ -1639,7 +1663,7 @@ TABLE *open_ltable(THD *thd, TABLE_LIST *table_list, thr_lock_type lock_type)
     {
       DBUG_ASSERT(thd->lock == 0);	// You must lock everything at once
       if ((table->reginfo.lock_type= lock_type) != TL_UNLOCK)
-	if (!(thd->lock=mysql_lock_tables(thd,&table_list->table,1)))
+	if (! (thd->lock= mysql_lock_tables(thd, &table_list->table, 1, 0)))
 	  table= 0;
     }
   }
@@ -1709,7 +1733,7 @@ int open_and_lock_tables(THD *thd, TABLE_LIST *tables)
   SYNOPSIS
     open_normal_and_derived_tables
     thd		- thread handler
-    tables	- list of tables for open&locking
+    tables	- list of tables for open
 
   RETURN
     FALSE - ok
@@ -1789,7 +1813,7 @@ int lock_tables(THD *thd, TABLE_LIST *tables, uint count)
       if (!table->derived)
 	*(ptr++)= table->table;
     }
-    if (!(thd->lock=mysql_lock_tables(thd,start, (uint) (ptr - start))))
+    if (! (thd->lock= mysql_lock_tables(thd, start, (uint) (ptr - start), 0)))
       return -1;				/* purecov: inspected */
   }
   else
@@ -1988,7 +2012,7 @@ find_field_in_tables(THD *thd, Item_ident *item, TABLE_LIST *tables,
   const char *name=item->field_name;
   uint length=(uint) strlen(name);
   char name_buff[NAME_LEN+1];
-
+  bool allow_rowid;
 
   if (item->cached_table)
   {
@@ -2080,7 +2104,7 @@ find_field_in_tables(THD *thd, Item_ident *item, TABLE_LIST *tables,
 	return (Field*) not_found_field;
     return (Field*) 0;
   }
-  bool allow_rowid= tables && !tables->next;	// Only one table
+  allow_rowid= tables && !tables->next;         // Only one table
   for (; tables ; tables=tables->next)
   {
     if (!tables->table)
@@ -2109,7 +2133,7 @@ find_field_in_tables(THD *thd, Item_ident *item, TABLE_LIST *tables,
 			name,thd->where);
 	return (Field*) 0;
       }
-      found=field;
+      found= field;
     }
   }
   if (found)
@@ -2169,6 +2193,7 @@ find_item_in_list(Item *find, List<Item> &items, uint *counter,
   bool found_unaliased_non_uniq= 0;
   uint unaliased_counter;
 
+  LINT_INIT(unaliased_counter);
   *unaliased= FALSE;
 
   if (find->type() == Item::FIELD_ITEM	|| find->type() == Item::REF_ITEM)
@@ -2864,8 +2889,18 @@ static void mysql_rm_tmp_tables(void)
 ** and afterwards delete those marked unused.
 */
 
-void remove_db_from_cache(const my_string db)
+void remove_db_from_cache(const char *db)
 {
+  char name_buff[NAME_LEN+1];
+  if (db && lower_case_table_names)
+  {
+    /*
+      convert database to lower case for comparision.
+    */
+    strmake(name_buff, db, sizeof(name_buff)-1);
+    my_casedn_str(files_charset_info, name_buff);
+    db= name_buff;
+  }
   for (uint idx=0 ; idx < open_cache.records ; idx++)
   {
     TABLE *table=(TABLE*) hash_element(&open_cache,idx);
@@ -2900,6 +2935,9 @@ void flush_tables()
   The table will be closed (not stored in cache) by the current thread when
   close_thread_tables() is called.
 
+  PREREQUISITES
+    Lock on LOCK_open()
+
   RETURN
     0  This thread now have exclusive access to this table and no other thread
        can access the table until close_thread_tables() is called.
@@ -2907,62 +2945,97 @@ void flush_tables()
 */
 
 bool remove_table_from_cache(THD *thd, const char *db, const char *table_name,
-			     bool return_if_owned_by_thd)
+                             uint flags)
 {
   char key[MAX_DBKEY_LENGTH];
   uint key_length;
   TABLE *table;
-  bool result=0;
+  bool result=0, signalled= 0;
   DBUG_ENTER("remove_table_from_cache");
 
+
   key_length=(uint) (strmov(strmov(key,db)+1,table_name)-key)+1;
-  for (table=(TABLE*) hash_search(&open_cache,(byte*) key,key_length) ;
-       table;
-       table = (TABLE*) hash_next(&open_cache,(byte*) key,key_length))
+  for (;;)
   {
-    THD *in_use;
-    table->version=0L;			/* Free when thread is ready */
-    if (!(in_use=table->in_use))
+    result= signalled= 0;
+
+    for (table=(TABLE*) hash_search(&open_cache,(byte*) key,key_length) ;
+         table;
+         table = (TABLE*) hash_next(&open_cache,(byte*) key,key_length))
     {
-      DBUG_PRINT("info",("Table was not in use"));
-      relink_unused(table);
-    }
-    else if (in_use != thd)
-    {
-      in_use->some_tables_deleted=1;
-      if (table->db_stat)
-	result=1;
-      /* Kill delayed insert threads */
-      if ((in_use->system_thread & SYSTEM_THREAD_DELAYED_INSERT) &&
-          ! in_use->killed)
+      THD *in_use;
+      table->version=0L;		/* Free when thread is ready */
+      if (!(in_use=table->in_use))
       {
-	in_use->killed=1;
-	pthread_mutex_lock(&in_use->mysys_var->mutex);
-	if (in_use->mysys_var->current_cond)
-	{
-	  pthread_mutex_lock(in_use->mysys_var->current_mutex);
-	  pthread_cond_broadcast(in_use->mysys_var->current_cond);
-	  pthread_mutex_unlock(in_use->mysys_var->current_mutex);
-	}
-	pthread_mutex_unlock(&in_use->mysys_var->mutex);
+        DBUG_PRINT("info",("Table was not in use"));
+        relink_unused(table);
       }
-      /*
-	Now we must abort all tables locks used by this thread
-	as the thread may be waiting to get a lock for another table
-      */
-      for (TABLE *thd_table= in_use->open_tables;
-	   thd_table ;
-	   thd_table= thd_table->next)
+      else if (in_use != thd)
       {
-	if (thd_table->db_stat)			// If table is open
-	  mysql_lock_abort_for_thread(thd, thd_table);
+        in_use->some_tables_deleted=1;
+        if (table->db_stat)
+  	  result=1;
+        /* Kill delayed insert threads */
+        if ((in_use->system_thread & SYSTEM_THREAD_DELAYED_INSERT) &&
+            ! in_use->killed)
+        {
+  	  in_use->killed=1;
+	  pthread_mutex_lock(&in_use->mysys_var->mutex);
+	  if (in_use->mysys_var->current_cond)
+	  {
+	    pthread_mutex_lock(in_use->mysys_var->current_mutex);
+            signalled= 1;
+	    pthread_cond_broadcast(in_use->mysys_var->current_cond);
+	    pthread_mutex_unlock(in_use->mysys_var->current_mutex);
+	  }
+	  pthread_mutex_unlock(&in_use->mysys_var->mutex);
+        }
+        /*
+	  Now we must abort all tables locks used by this thread
+	  as the thread may be waiting to get a lock for another table
+        */
+        for (TABLE *thd_table= in_use->open_tables;
+	     thd_table ;
+	     thd_table= thd_table->next)
+        {
+	  if (thd_table->db_stat)		// If table is open
+	    signalled|= mysql_lock_abort_for_thread(thd, thd_table);
+        }
+      }
+      else
+        result= result || (flags & RTFC_OWNED_BY_THD_FLAG);
+    }
+    while (unused_tables && !unused_tables->version)
+      VOID(hash_delete(&open_cache,(byte*) unused_tables));
+    if (result && (flags & RTFC_WAIT_OTHER_THREAD_FLAG))
+    {
+      if (!(flags & RTFC_CHECK_KILLED_FLAG) || !thd->killed)
+      {
+        dropping_tables++;
+        if (likely(signalled))
+          (void) pthread_cond_wait(&COND_refresh, &LOCK_open);
+        else
+        {
+          struct timespec abstime;
+          /*
+            It can happen that another thread has opened the
+            table but has not yet locked any table at all. Since
+            it can be locked waiting for a table that our thread
+            has done LOCK TABLE x WRITE on previously, we need to
+            ensure that the thread actually hears our signal
+            before we go to sleep. Thus we wait for a short time
+            and then we retry another loop in the
+            remove_table_from_cache routine.
+          */
+          set_timespec(abstime, 10);
+          pthread_cond_timedwait(&COND_refresh, &LOCK_open, &abstime);
+        }
+        dropping_tables--;
+        continue;
       }
     }
-    else
-      result= result || return_if_owned_by_thd;
+    break;
   }
-  while (unused_tables && !unused_tables->version)
-    VOID(hash_delete(&open_cache,(byte*) unused_tables));
   DBUG_RETURN(result);
 }
 
