@@ -363,6 +363,7 @@ struct system_variables
   ulong max_insert_delayed_threads;
   ulong myisam_repair_threads;
   ulong myisam_sort_buff_size;
+  ulong myisam_stats_method;
   ulong net_buffer_length;
   ulong net_interactive_timeout;
   ulong net_read_timeout;
@@ -916,7 +917,7 @@ public:
   bool       query_error, bootstrap, cleanup_done;
   bool	     tmp_table_used;
   bool	     charset_is_system_charset, charset_is_collation_connection;
-  bool       slow_command;
+  bool       enable_slow_log;   /* enable slow log for current statement */
   my_bool    volatile killed;
 
   /*
@@ -1236,19 +1237,27 @@ class select_insert :public select_result_interceptor {
   List<Item> *fields;
   ulonglong last_insert_id;
   COPY_INFO info;
+  TABLE_LIST *insert_table_list;
+  TABLE_LIST *dup_table_list;
 
   select_insert(TABLE *table_par, List<Item> *fields_par,
 		enum_duplicates duplic, bool ignore)
-    :table(table_par), fields(fields_par), last_insert_id(0)
+    :table(table_par), fields(fields_par), last_insert_id(0),
+     insert_table_list(0), dup_table_list(0)
   {
     bzero((char*) &info,sizeof(info));
     info.ignore= ignore;
     info.handle_duplicates=duplic;
   }
-  select_insert(TABLE *table_par, List<Item> *fields_par,
+  select_insert(TABLE *table_par,
+		TABLE_LIST *insert_table_list_par,
+		TABLE_LIST *dup_table_list_par,
+		List<Item> *fields_par,
 		List<Item> *update_fields, List<Item> *update_values,
 		enum_duplicates duplic, bool ignore)
-    :table(table_par), fields(fields_par), last_insert_id(0)
+    :table(table_par), fields(fields_par), last_insert_id(0),
+     insert_table_list(insert_table_list_par),
+     dup_table_list(dup_table_list_par)
   {
     bzero((char*) &info,sizeof(info));
     info.ignore= ignore;
