@@ -260,26 +260,29 @@ uint my_charpos_mb(CHARSET_INFO *cs __attribute__((unused)),
     pos+= (mblen= my_ismbchar(cs, pos, end)) ? mblen : 1;
     length--;
   }
-  return length ? end+2-start : pos-start;
+  return length ? (uint) (end + 2 - start) : (uint) (pos - start);
 }
 
 
-uint my_well_formed_len_mb(CHARSET_INFO *cs,
-			   const char *b, const char *e, uint pos)
+uint my_well_formed_len_mb(CHARSET_INFO *cs, const char *b, const char *e,
+                           uint pos, int *error)
 {
   const char *b_start= b;
-  
+  *error= 0;
   while (pos)
   {
     my_wc_t wc;
     int mblen;
 
     if ((mblen= cs->cset->mb_wc(cs, &wc, (uchar*) b, (uchar*) e)) <= 0)
+    {
+      *error= b < e ? 1 : 0;
       break;
+    }
     b+= mblen;
     pos--;
   }
-  return b - b_start;
+  return (uint) (b - b_start);
 }
 
 
@@ -499,17 +502,13 @@ my_bool my_like_range_mb(CHARSET_INFO *cs,
 			 char *min_str,char *max_str,
 			 uint *min_length,uint *max_length)
 {
-  const char *end;
+  const char *end= ptr + ptr_length;
   char *min_org= min_str;
   char *min_end= min_str + res_length;
   char *max_end= max_str + res_length;
-  uint charlen= my_charpos(cs, ptr, ptr+ptr_length, res_length/cs->mbmaxlen);
+  uint charlen= res_length / cs->mbmaxlen;
 
-  if (charlen < ptr_length)
-    ptr_length= charlen;
-  end= ptr + ptr_length;
-
-  for (; ptr != end && min_str != min_end ; ptr++)
+  for (; ptr != end && min_str != min_end && charlen > 0 ; ptr++, charlen--)
   {
     if (*ptr == escape && ptr+1 != end)
     {
@@ -519,14 +518,8 @@ my_bool my_like_range_mb(CHARSET_INFO *cs,
     }
     if (*ptr == w_one || *ptr == w_many)	/* '_' and '%' in SQL */
     {
-      charlen= my_charpos(cs, min_org, min_str, res_length/cs->mbmaxlen);
-      
-      if (charlen < (uint) (min_str - min_org))
-        min_str= min_org + charlen;
-      
       /* Write min key  */
       *min_length= (uint) (min_str - min_org);
-      *max_length= res_length;
       do
       {
 	*min_str++= (char) cs->min_sort_char;
@@ -537,6 +530,7 @@ my_bool my_like_range_mb(CHARSET_INFO *cs,
         representation of the max_sort_char character,
         and copy it into max_str in a loop. 
       */
+      *max_length= res_length;
       pad_max_char(cs, max_str, max_end);
       return 0;
     }
