@@ -997,11 +997,12 @@ my_bool my_like_range_simple(CHARSET_INFO *cs,
 			     char *min_str,char *max_str,
 			     uint *min_length,uint *max_length)
 {
-  const char *end=ptr+ptr_length;
+  const char *end= ptr + ptr_length;
   char *min_org=min_str;
   char *min_end=min_str+res_length;
+  uint charlen= res_length / cs->mbmaxlen;
 
-  for (; ptr != end && min_str != min_end ; ptr++)
+  for (; ptr != end && min_str != min_end && charlen > 0 ; ptr++, charlen--)
   {
     if (*ptr == escape && ptr+1 != end)
     {
@@ -1045,7 +1046,7 @@ ulong my_scan_8bit(CHARSET_INFO *cs, const char *str, const char *end, int sq)
     if (*str == '.')
     {
       for(str++ ; str != end && *str == '0' ; str++);
-      return str-str0;
+      return (ulong) (str - str0);
     }
     return 0;
 
@@ -1055,7 +1056,7 @@ ulong my_scan_8bit(CHARSET_INFO *cs, const char *str, const char *end, int sq)
       if (!my_isspace(cs,*str))
         break;
     }
-    return str-str0;
+    return (ulong) (str - str0);
   default:
     return 0;
   }
@@ -1072,14 +1073,14 @@ void my_fill_8bit(CHARSET_INFO *cs __attribute__((unused)),
 uint my_numchars_8bit(CHARSET_INFO *cs __attribute__((unused)),
 		      const char *b, const char *e)
 {
-  return e-b;
+  return (uint) (e - b);
 }
 
 
 uint my_numcells_8bit(CHARSET_INFO *cs __attribute__((unused)),
 		      const char *b, const char *e)
 {
-  return e-b;
+  return (uint) (e - b);
 }
 
 
@@ -1093,11 +1094,11 @@ uint my_charpos_8bit(CHARSET_INFO *cs __attribute__((unused)),
 
 
 uint my_well_formed_len_8bit(CHARSET_INFO *cs __attribute__((unused)),
-			     const char *start,
-			     const char *end,
-			     uint nchars)
+                             const char *start, const char *end,
+                             uint nchars, int *error)
 {
   uint nbytes= (uint) (end-start);
+  *error= 0;
   return min(nbytes, nchars);
 }
 
