@@ -209,13 +209,13 @@ static int my_strnncoll_ucs2(CHARSET_INFO *cs,
     t_wc = uni_plane[plane] ? uni_plane[plane][t_wc & 0xFF].sort : t_wc;
     if ( s_wc != t_wc )
     {
-      return  ((int) s_wc) - ((int) t_wc);
+      return  s_wc > t_wc ? 1 : -1;
     }
     
     s+=s_res;
     t+=t_res;
   }
-  return t_is_prefix ? t-te : ((se-s) - (te-t));
+  return t_is_prefix ? (int) (t - te) : (int) ((se - s) - (te - t));
 }
 
 /*
@@ -267,7 +267,7 @@ static int my_strnncollsp_ucs2(CHARSET_INFO *cs __attribute__((unused)),
     int t_wc = uni_plane[t[0]] ? (int) uni_plane[t[0]][t[1]].sort : 
                                  (((int) t[0]) << 8) + (int) t[1];
     if ( s_wc != t_wc )
-      return  s_wc - t_wc;
+      return  s_wc > t_wc ? 1 : -1;
 
     s+= 2;
     t+= 2;
@@ -326,7 +326,7 @@ static int my_strncasecmp_ucs2(CHARSET_INFO *cs,
     s+=s_res;
     t+=t_res;
   }
-  return ( (se-s) - (te-t) );
+  return (int) ( (se-s) - (te-t) );
 }
 
 
@@ -1251,7 +1251,7 @@ static
 uint my_numchars_ucs2(CHARSET_INFO *cs __attribute__((unused)),
 		      const char *b, const char *e)
 {
-  return (e-b)/2;
+  return (uint) (e-b)/2;
 }
 
 
@@ -1261,7 +1261,8 @@ uint my_charpos_ucs2(CHARSET_INFO *cs __attribute__((unused)),
 		     const char *e  __attribute__((unused)),
 		     uint pos)
 {
-  return pos*2;
+  uint string_length= (uint) (e - b);
+  return pos > string_length ? string_length + 2 : pos * 2;
 }
 
 
@@ -1270,7 +1271,8 @@ uint my_well_formed_len_ucs2(CHARSET_INFO *cs __attribute__((unused)),
                              const char *b, const char *e,
                              uint nchars, int *error)
 {
-  uint nbytes= (e-b) & ~ (uint)1;
+  /* Ensure string length is dividable with 2 */
+  uint nbytes= ((uint) (e-b)) & ~(uint) 1;
   *error= 0;
   nchars*= 2;
   return min(nbytes, nchars);
@@ -1341,13 +1343,13 @@ int my_strnncoll_ucs2_bin(CHARSET_INFO *cs,
     }
     if ( s_wc != t_wc )
     {
-      return  ((int) s_wc) - ((int) t_wc);
+      return  s_wc > t_wc ? 1 : -1;
     }
     
     s+=s_res;
     t+=t_res;
   }
-  return t_is_prefix ? t-te : ((se-s) - (te-t));
+  return t_is_prefix ? (int) (t - te) : (int) ((se-s) - (te-t));
 }
 
 static int my_strnncollsp_ucs2_bin(CHARSET_INFO *cs, 
@@ -1424,10 +1426,12 @@ my_bool my_like_range_ucs2(CHARSET_INFO *cs,
   const char *end=ptr+ptr_length;
   char *min_org=min_str;
   char *min_end=min_str+res_length;
+  uint charlen= res_length / cs->mbmaxlen;
   
-  for (; ptr + 1 < end && min_str + 1 < min_end ; ptr+=2)
+  for ( ; ptr + 1 < end && min_str + 1 < min_end && charlen > 0
+        ; ptr+=2, charlen--)
   {
-    if (ptr[0] == '\0' && ptr[1] == escape && ptr+2 < end)
+    if (ptr[0] == '\0' && ptr[1] == escape && ptr + 1 < end)
     {
       ptr+=2;					/* Skip escape */
       *min_str++= *max_str++ = ptr[0];
@@ -1492,7 +1496,7 @@ ulong my_scan_ucs2(CHARSET_INFO *cs __attribute__((unused)),
       if (str[0] != '\0' || str[1] != ' ')
         break;
     }
-    return str - str0;
+    return (ulong) (str - str0);
   default:
     return 0;
   }
@@ -1581,6 +1585,7 @@ CHARSET_INFO my_charset_ucs2_general_ci=
     2,			/* mbmaxlen     */
     0,			/* min_sort_char */
     0xFFFF,		/* max_sort_char */
+    0,                  /* escape_with_backslash_is_dangerous */
     &my_charset_ucs2_handler,
     &my_collation_ucs2_general_ci_handler
 };
@@ -1608,6 +1613,7 @@ CHARSET_INFO my_charset_ucs2_bin=
     2,			/* mbmaxlen     */
     0,			/* min_sort_char */
     0xFFFF,		/* max_sort_char */
+    0,                  /* escape_with_backslash_is_dangerous */
     &my_charset_ucs2_handler,
     &my_collation_ucs2_bin_handler
 };
